/*********************************************************************
 * 'PataCal 1.0.0: The pataPhysical calendar
 * (C) Jrme Euzenat, 2000-2001
 *********************************************************************/

#include "patacal.h"
#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#define CREATOR		'PtCl'

#define ERROR -1
#define TRUE 1
#define FALSE 0
#define MAIN 0

#define HUNDAYI 6	// Index of the gidouille month in the table
#define BISEXT 1	// Index of the gueule month in the table
#define MAXDATE 256

DateType DisplayDate;	// Vulgar date
Int mm, dd, yy;		// Pataphysical date
Boolean leap;		// Leap vulgar year
ULong dy;		// number of days since the begining of the vulgar year
char str[MAXDATE];

#define ComputeYear(yy) yy+31
// yy+1904-31

#define mlen(x) (x == HUNDAYI || (leap && x== BISEXT) )?29:28

#define hunyadi() dd==29&&(mm!=HUNDAYI)&&(mm!=BISEXT||!leap)

#define leapyearp(year) (!((year)%4) && ( (year)%100 || !((year)%400) ))

void SetText(FieldPtr fieldPtr, CharPtr text)
{
    VoidHand textH;
    CharPtr textP;

    textH = MemHandleNew(StrLen(text)+1);
    if (textH == NULL) return;
    textP = MemHandleLock(textH);
    StrCopy(textP, text);
    MemHandleUnlock(textH);
    FldSetTextHandle(fieldPtr, (Handle)textH);
}

void ComputePataData(DateType date)
{
    DateType dt;

    // Leap year?
    leap = leapyearp( date.year+1904 );

    // adjust Pataphysical year (begins the 8th of september 1873)
    yy = ComputeYear(date.year); 
    if( date.month > 9 || (date.month==9 && date.day>7)) yy++;
    // Days in the year
    dt.year = date.year; dt.month = 1; dt.day = 1;
    dy = DateToDays(date) - DateToDays(dt) + 1;
    // Month and Day
    mm=0;
    if ( dy<363 || (dy==363 && leap)) {
	dd = (Int)dy + 3; // The 3 days of end of the previous year
	while ( dd > (mlen(mm)) ) { dd=dd-(mlen(mm)); mm++;};
    } else { dd = (Int)dy - (leap?363:362); };
}

#define NBDAYSSINCE1ABS 115 //=23+31+30+31

void ComputeVulgaData(Int dd, Int mm, Int yy)
{ // dy is used in this function as an auxiliary variable and reset in the end
    DateType dt;
    Int rm = (mm+4)%13;
    
    // Leap year? +1 because it is only useful on gidouille, second part of the year
    leap = leapyearp( yy+(((rm<4)||((rm==4)&&(dd<4)))?1872:1873) );

    // number of days since the begining of the pataphysical year
    dy = (rm)*28 + ((rm>5 && leap)?1:0) + ((rm>10)?1:0) + dd -1; ///LEAPON(+1)
    if( dy >= NBDAYSSINCE1ABS ) {
      dy = dy-NBDAYSSINCE1ABS; dt.year = yy-31;
    } else {
      dy = (leap?251:250)+dy; dt.year = yy-32; ///LEAPON(-1)
    }
    if( !leap ) dy=dy+366; //1904 was bisextile
    DateDaysToDate( dy , &DisplayDate );
    DisplayDate.year = dt.year;

    // Days in the year must also be recomputed
    dt.month = 1; dt.day = 1;
    dy = DateToDays(DisplayDate) - DateToDays(dt) + 1;
}

// Called only when DisplayDate, yy, mm, dd and leap are defined
void DisplayPanel() 
{
    FormPtr f = FrmGetActiveForm();
    
    static char *mois[13] = {
        "dcervelage",
        "gueules",
        "pdale",
        "clinamen",
        "palotin",
        "merdre",
        "gidouille",
        "tatane",
        "phalle",
        "absolu",
        "haha",
        "as",
        "sable"
    };
    
#ifdef SAINTS
#include "patasts.h"
#endif

    // Vulgar date
    if ( hunyadi() ) {
	SetText((FieldPtr)FrmGetObjectPtr(f, FrmGetObjectIndex(f, dayField)), "Hunyadi imaginaire");
    } else {
	DateToDOWDMFormat(DisplayDate.month,DisplayDate.day,DisplayDate.year+1904,TRUE,str);
	SetText((FieldPtr)FrmGetObjectPtr(f, FrmGetObjectIndex(f, dayField)), str);
    };

    // Pataphysical date
    StrCopy(str,"");
    StrIToA(str + StrLen(str), dd);
    StrCat(str, " ");
    StrCat(str,  mois[mm]);
    StrCat(str, " ");
    StrIToA(str + StrLen(str),  yy);
    SetText((FieldPtr)FrmGetObjectPtr(f, FrmGetObjectIndex(f, dateField)), str);
    
    // Saint
#ifdef SAINTS
    if ( hunyadi() ) { StrCopy(str,"Hunyadi"); // not yet...
    } else if ( leap && dy>=54 ) { // 54 is february 23rd
	if ( dy==54 ) StrCopy(str,"Hunyadi, Mouvement perptuel");
	else StrCopy(str,saints[(Int)dy-2]);
    // The following could be achieved by a table of Hunyadis
    } else {
      StrCopy(str,saints[(Int)dy-1]);
    };
    SetText((FieldPtr)FrmGetObjectPtr(f, FrmGetObjectIndex(f, saintField)), str);
#endif
    FrmDrawForm(f);
}

void IncText( FieldPtr field, Int i ){
    Long val;
    VoidHand textH;
    CharPtr textP, str = (CharPtr)MemPtrNew(16);

    textH = (VoidHand)FldGetTextHandle(field);
    if (textH == NULL) return;
    textP = MemHandleLock(textH);
    StrCopy(str, textP);
    if (str == NULL) return;
    if (StrLen(str) == 0) { MemPtrFree(str); return; }
    StrIToA(str, i+StrAToI(str));
    MemHandleUnlock(textH);
    textH = MemHandleNew(StrLen(str)+1);
    if (textH == NULL) return;
    textP = MemHandleLock(textH);
    StrCopy(textP, str);
    MemHandleUnlock(textH);
    FldSetTextHandle(field, (Handle)textH);
    MemPtrFree(str);
}

void DecDate()
{ if( dd>1 ) dd--;
  else if(mm!=9) {
     mm=(mm+12)%13; dd = 29;
  } else {
     yy--; mm=8; dd=29;
  };
  ComputeVulgaData(dd, mm, yy);
  DisplayPanel();
}
	
void IncDate()
{ if (dd==29 && mm==8) {
    dd=1; mm=9; yy++;
  } else if (dd==29) {
    dd=1; mm=(mm+1)%13;
  } else dd++;
  ComputeVulgaData(dd, mm, yy);
  DisplayPanel();
}

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
    EventType evt;
    Word error;
    FormPtr f;
    
    if (cmd != sysAppLaunchCmdNormalLaunch) return FALSE;

    DateSecondsToDate(TimGetSeconds(), &DisplayDate);
    FrmGotoForm(mainForm);

    while (TRUE)
    {
	EvtGetEvent(&evt, evtWaitForever);
	if (SysHandleEvent(&evt)) continue;
	if (MenuHandleEvent(NULL, &evt, &error)) continue;

	switch (evt.eType)
	{
	  case ctlSelectEvent:
	    switch (evt.data.ctlSelect.controlID)
	    {
	      case okAboutBtn:
		FrmReturnToForm(mainForm);
		break;

	      case dateBtn:
		{ // Momentaneous particular use of dd/mm/yy
		  dd = DisplayDate.day;
		  mm = DisplayDate.month;
		  yy = DisplayDate.year + 1904;
                  SelectDay(selectDayByDay,&mm, &dd, &yy, "Select Date");
		  DisplayDate.year = yy - 1904;
		  DisplayDate.month = mm;
		  DisplayDate.day = dd;
		  ComputePataData(DisplayDate);
		  // End of critical section
		  DisplayPanel();};
		break;

	      case chooseBtn:
		{ FrmPopupForm(choosePDateForm); };
		break;

	      case precYear:
		yy = -1; // no break
	      case nextYear:
		{ f = FrmGetActiveForm();
		  IncText( (FieldPtr)FrmGetObjectPtr(f, FrmGetObjectIndex(f, yearField)), (yy)?yy:1);
		  yy=0;
		  FrmDrawForm(f); };
		break;

	      case befBtn: DecDate(); break;
	      case aftBtn: IncDate(); break;

	      case okChooseBtn:
		{ CharPtr s;
                  VoidHand textH = (VoidHand)FldGetTextHandle((FieldPtr)FrmGetObjectPtr(f, FrmGetObjectIndex(f, yearField)));
                  if (!textH) return ERROR;
                  s = MemHandleLock(textH);
		  if (!StrLen(s))return ERROR;
                  yy = StrAToI(s);
                  MemHandleUnlock(textH);
		  mm = FrmGetObjectId( f, FrmGetControlGroupSelection( f, monthGroup)) - 1500 - 1;
		  dd = FrmGetObjectId( f, FrmGetControlGroupSelection( f, dayGroup )) - 1550 ;
		  ComputeVulgaData(dd, mm, yy);
                  FrmReturnToForm(mainForm);
		  DisplayPanel();
		}
		break;
		
	      case todayBtn:
		{
		  DateSecondsToDate(TimGetSeconds(), &DisplayDate);
		  ComputePataData(DisplayDate);
                  FrmReturnToForm(mainForm);
		  DisplayPanel();
		}
		break;
		
	      default:
		FrmHandleEvent(FrmGetActiveForm(), &evt);
	    }
	    break;
	  
	  case keyDownEvent:
	    if ( evt.data.keyDown.chr == pageUpChr ) IncDate();
	    else if ( evt.data.keyDown.chr == pageDownChr ) DecDate();
	    break;

	  case frmLoadEvent:
	    FrmSetActiveForm(FrmInitForm(evt.data.frmLoad.formID));
	    break;

	  case frmOpenEvent:
	    if ( evt.data.frmOpen.formID == mainForm){
		ComputePataData(DisplayDate);
		DisplayPanel();
	    } else if (evt.data.frmOpen.formID == choosePDateForm){
	    	f = FrmGetActiveForm();
	        SetText((FieldPtr)FrmGetObjectPtr(f, FrmGetObjectIndex(f, yearField)), StrIToA(str, yy));
		FrmSetControlGroupSelection( f, dayGroup, dd + 1550 );
		FrmSetControlGroupSelection( f, monthGroup, mm + 1500 + 1 );
		// During the activity time of this panel, yy is used as an indicator
		// of how the year must be incremented (incr/decr)
		// Its value is restored at output time
		yy=0;
	    };
	    FrmDrawForm(FrmGetActiveForm());
	    break;

	  case menuEvent:
	    switch ( evt.data.menu.itemID ) {
		case menuItemAbout: FrmPopupForm(aboutForm) ; break;
		case menuItemCopy: 
	 	      f = FrmGetActiveForm();
		      {  FieldPtr field = (FieldPtr)FrmGetObjectPtr(f, FrmGetObjectIndex(f, dateField));
		         FldSetSelection( field, 0, FldGetTextLength( field ));
        	         FldCopy(field);
		         FldSetSelection( field, 0, 0);
		break;}
		default: FrmHandleEvent(FrmGetActiveForm(), &evt);
		}
	    break;

	  case appStopEvent:
	    FrmCloseAllForms();
	    return FALSE;

	  default:
	    FrmHandleEvent(FrmGetActiveForm(), &evt);
	}
    }
    return FALSE;
}

